@@ -27,14 +27,64 @@
 //! | Long | `i64` or `u64` | use owned |
 //! | Float | `f32` | use owned |
 //! | Double | `f64` | use owned |
-//! | String | `&str` | `String` |
+//! | String | `String` | [`Cow<'a, str>`][`std::borrow::Cow`] (see below) |
 //! | List | `Vec<T>` | use owned |
 //! | Byte Array | [`ByteArray`][`crate::ByteArray`] | [`borrow::ByteArray`][`crate::borrow::ByteArray`] |
 //! | Int Array | [`IntArray`][`crate::IntArray`] | [`borrow::IntArray`][`crate::borrow::IntArray`] |
 //! | Long Array | [`LongArray`][`crate::LongArray`] | [`borrow::LongArray`][`crate::borrow::LongArray`] |
 //!
-//! Borrowing for types like `i64`, `u32`, or `f64` is generally not possible
-//! due to alignment requirements. It likely wouldn't be faster/smaller anyway.
+//! ## Primitives
+//!
+//! Borrowing for primitive types like the integers and floats is generally not
+//! possible due to alignment requirements of those types. It likely wouldn't be
+//! faster/smaller anyway.
+//!
+//! ## Strings
+//!
+//! For strings, we cannot know ahead of time whether the data can be borrowed
+//! as `&str`. This is because Minecraft uses Java's encoding of Unicode, which
+//! is not UTF-8 like Rust. If the string contains Unicode characters outside of
+//! the Basic Multilingual Plane then we need to convert it to utf-8, requiring
+//! us to own the string data.
+//!
+//! Using [`Cow<'a, str>`][`std::borrow::Cow`] lets us borrow when possible, but
+//! produce an owned value when the representation is different. This will be
+//! common for minecrafts internal strings and any world whose language falls in
+//! the basic multilingual plane.
+//!
+//! In future we could support a lazy string type that always borrows the
+//! underyling data and decodes when needed. Please open an issue if this is
+//! important to you.
+//!
+//! ## Representation of NBT arrays
+//!
+//! In order for [`Value`][`crate::Value`] to preserve all NBT information, the
+//! deserializer "[maps into serde's data
+//! model](https://serde.rs/data-model.html#mapping-into-the-data-model)". This
+//! means that the NBT arrays come out not as a simple 'seq' but instead
+//! something like
+//!
+//! ```txt
+//! map{
+//!     tag: u8,
+//!     data: seq<T>,
+//! }
+//! ```
+//!
+//! in some pseudo-language for serde's data model.
+//!
+//! These means the Int Array can be modelled something like this in Rust:
+//!
+//! ```rust
+//! pub struct IntArray {
+//!    tag: u8,
+//!    data: Vec<i32>,
+//! }
+//! ```
+//!
+//! The actual structure is slightly different in order to check the tag is
+//! correct. You can create your own types to capture these Arrays or just use
+//! the ones provided in this crate.
 //!
 //! # Other quirks
 //!
@@ -187,6 +237,7 @@
 //! }
 //! ```
 
+use std::borrow::Cow;
 use std::convert::{TryFrom, TryInto};
 use std::ops::Range;
 
@@ -222,7 +273,7 @@ pub fn from_bytes<'a, T>(input: &'a [u8]) -> Result<T>
 where
     T: de::Deserialize<'a>,
 {
-    let mut des = Deserializer::from_bytes(&input);
+    let mut des = Deserializer::from_bytes(input);
     let t = T::deserialize(&mut des)?;
     Ok(t)
 }
@@ -273,6 +324,16 @@ enum Layer {
 /// to borrow just the input, making us free to also borrow/mutate the layers.
 pub(crate) struct InputHelper<'de>(pub(crate) &'de [u8]);
 
+fn visit_cow_str<'de, V>(v: V, s: Cow<'de, str>) -> Result<V::Value>
+where
+    V: de::Visitor<'de>,
+{
+    match s {
+        Cow::Borrowed(s) => v.visit_borrowed_str(s),
+        Cow::Owned(s) => v.visit_string(s),
+    }
+}
+
 fn consume_value<'de, V>(de: &mut Deserializer<'de>, visitor: V, tag: Tag) -> Result<V::Value>
 where
     V: de::Visitor<'de>,
@@ -285,7 +346,7 @@ where
         Tag::Short => visitor.visit_i16(de.input.0.read_i16::<BigEndian>()?),
         Tag::Int => visitor.visit_i32(de.input.0.read_i32::<BigEndian>()?),
         Tag::Long => visitor.visit_i64(de.input.0.read_i64::<BigEndian>()?),
-        Tag::String => visitor.visit_borrowed_str(de.input.consume_size_prefixed_string()?),
+        Tag::String => visit_cow_str(visitor, de.input.consume_size_prefixed_string()?),
         Tag::Float => visitor.visit_f32(de.input.consume_float()?),
         Tag::Double => visitor.visit_f64(de.input.consume_double()?),
         Tag::Compound => {
@@ -352,19 +413,21 @@ impl<'de> InputHelper<'de> {
 
     fn consume_tag(&mut self) -> Result<Tag> {
         let tag_byte = self.0.read_u8()?;
-        Tag::try_from(tag_byte).or_else(|_| Err(Error::invalid_tag(tag_byte)))
+        Tag::try_from(tag_byte).map_err(|_| Error::invalid_tag(tag_byte))
     }
 
-    fn consume_name(&mut self) -> Result<&'de str> {
+    fn consume_name(&mut self) -> Result<Cow<'de, str>> {
         self.consume_size_prefixed_string()
     }
 
-    fn consume_size_prefixed_string(&mut self) -> Result<&'de str> {
+    fn consume_size_prefixed_string(&mut self) -> Result<Cow<'de, str>> {
         let len = self.0.read_u16::<BigEndian>()? as usize;
-        let s = std::str::from_utf8(self.subslice(0..len)?)
-            .map_err(|_| Error::nonunicode_string(&self.0[..len]));
+        let str_data = self.subslice(0..len)?;
+        let s = cesu8::from_java_cesu8(str_data)
+            .map_err(|_| Error::nonunicode_string(&self.0[..len]))?;
+
         self.0 = &self.0[len..];
-        s
+        Ok(s)
     }
 
     fn consume_bytes_unchecked(&mut self, size: i32) -> Result<&'de [u8]> {
@@ -500,11 +563,11 @@ impl<'de, 'a> de::Deserializer<'de> for &'a mut Deserializer<'de> {
                         Stage::Tag => {
                             *current_tag = Some(self.input.consume_tag()?);
                             *stage = Stage::Value;
-                            return visitor.visit_borrowed_str(self.input.consume_name()?);
+                            return visit_cow_str(visitor, self.input.consume_name()?);
                         }
                         Stage::Name => {
                             *stage = Stage::Value;
-                            return visitor.visit_borrowed_str(self.input.consume_name()?);
+                            return visit_cow_str(visitor, self.input.consume_name()?);
                         }
                         Stage::Value => {
                             *stage = Stage::Tag;
@@ -563,9 +626,9 @@ impl<'de, 'a> de::Deserializer<'de> for &'a mut Deserializer<'de> {
     where
         V: de::Visitor<'de>,
     {
-        let layer = self.layers.last().ok_or(Error::bespoke(format!(
-            "expected bytes, but not in a compound or list",
-        )))?;
+        let layer = self.layers.last().ok_or_else(|| {
+            Error::bespoke("expected bytes, but not in a compound or list".to_owned())
+        })?;
 
         match layer {
             Layer::List {
@@ -723,13 +786,9 @@ impl<'de, 'a> de::Deserializer<'de> for &'a mut Deserializer<'de> {
         // The last layer should tell us what value we're expecting.
         // We have already read the tag and name. This is the payload.
 
-        let layer = self
-            .layers
-            .last()
-            .ok_or(Error::bespoke(format!(
-                "expected unwanted payload, but not in a compound or list",
-            )))?
-            .clone();
+        let layer = self.layers.last().ok_or_else(|| {
+            Error::bespoke("expected unwanted payload, but not in a compound or list".to_owned())
+        })?;
 
         match layer {
             Layer::Compound {
@@ -838,7 +897,7 @@ impl<'a, 'de> de::SeqAccess<'de> for ListAccess<'a, 'de> {
             .de
             .layers
             .last_mut()
-            .ok_or(Error::bespoke("expected to be in list".to_owned()))?;
+            .ok_or_else(|| Error::bespoke("expected to be in list".to_owned()))?;
 
         match layer {
             Layer::List {
@@ -846,7 +905,7 @@ impl<'a, 'de> de::SeqAccess<'de> for ListAccess<'a, 'de> {
                 element_tag: _,
             } => {
                 if *remaining_elements > 0 {
-                    *remaining_elements = *remaining_elements - 1;
+                    *remaining_elements -= 1;
                     let val = seed.deserialize(&mut *self.de)?;
                     Ok(Some(val))
                 } else {
